/******************************************************************************
 *    BowService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IBowSrv>::Name = "BowService";

	const char * 
	ScriptService<IBowSrv>::DocStrings[] = {
		"BowService()\n\t"
			"Provides an interface for manipulating the player bow arm.",
		"Equip() -> Integer\n\t"
			"Always returns 1.",
		"UnEquip() -> Integer\n\t"
			"Always returns 1.",
		"IsEquipped() -> Boolean",
		"StartAttack() -> Integer\n\t"
			"Always returns 1.",
		"FinishAttack() -> Integer\n\t"
			"Returns 1 on success, otherwise 0.",
		"AbortAttack() -> Integer\n\t"
			"Returns 1 on success, otherwise 0.",
		"SetArrow(obj) -> Integer\n\t"
			"Returns 1 on success, otherwise 0.",
	};

	const PyMethodDef 
	ScriptService<IBowSrv>::Methods[] = {
		{	"Equip",
			reinterpret_cast<PyCFunction>(Equip),
			METH_NOARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"UnEquip",
			reinterpret_cast<PyCFunction>(UnEquip),
			METH_NOARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"IsEquipped",
			reinterpret_cast<PyCFunction>(IsEquipped),
			METH_NOARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"StartAttack",
			reinterpret_cast<PyCFunction>(StartAttack),
			METH_NOARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"FinishAttack",
			reinterpret_cast<PyCFunction>(FinishAttack),
			METH_NOARGS,
			const_cast<char*>(DocStrings[5])
		},
		{	"AbortAttack",
			reinterpret_cast<PyCFunction>(AbortAttack),
			METH_NOARGS,
			const_cast<char*>(DocStrings[6])
		},
		{	"SetArrow",
			reinterpret_cast<PyCFunction>(SetArrow),
			METH_VARARGS,
			const_cast<char*>(DocStrings[7])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IBowSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IBowSrv>::Name),
		sizeof(ScriptServiceObject<IBowSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IBowSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IBowSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<IBowSrv> * 
	ScriptService<IBowSrv>::NewObject(PyObject *)
	{
		IBowSrv *iface = static_cast<IBowSrv*>(g_pScriptManager->GetService(IID_IBowScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IBowSrv> *obj = PyObject_New(ScriptServiceObject<IBowSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IBowSrv>::Equip(ScriptServiceObject<IBowSrv> *self)
	{
		long ret = self->m_pIFace->Equip();
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IBowSrv>::UnEquip(ScriptServiceObject<IBowSrv> *self)
	{
		long ret = self->m_pIFace->UnEquip();
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IBowSrv>::IsEquipped(ScriptServiceObject<IBowSrv> *self)
	{
		PyObject *ret = (self->m_pIFace->IsEquipped()) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}

	PyObject * 
	ScriptService<IBowSrv>::StartAttack(ScriptServiceObject<IBowSrv> *self)
	{
		long ret = self->m_pIFace->StartAttack();
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IBowSrv>::FinishAttack(ScriptServiceObject<IBowSrv> *self)
	{
		long ret = self->m_pIFace->FinishAttack();
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IBowSrv>::AbortAttack(ScriptServiceObject<IBowSrv> *self)
	{
		long ret = self->m_pIFace->AbortAttack();
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IBowSrv>::SetArrow(ScriptServiceObject<IBowSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:SetArrow", cvtObject, &obj))
		{
			return NULL;
		}
		int ret = self->m_pIFace->SetArrow(obj);
		return PyInt_FromLong(ret); // XXX: Is this an object ID or a truth value?
	}


} // namespace PyNumbra


