/******************************************************************************
 *    PyNumbraTypes.h
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include <lg/types.h>
#include <lg/scrmanagers.h>

namespace PyNumbra
{

	//extern CInterface<ILinkManager> CLinkManager;
	//extern CInterface<IObjectSystem> CObjectSystem;

	class ObjectObject : public PyObject
	{
	public:
		object m_object;
		
		static void InitModule(PyObject *module);
		static PyObject * New(object o);
		static PyObject * Pickle(PyObject *nil, ObjectObject *self);
		static PyObject * Construct(PyTypeObject *type, PyObject *args, PyObject *kwds);
		static void Dealloc(ObjectObject *self);

		static PyObject * Repr(ObjectObject *self);
		static PyObject * Str(ObjectObject *self);
		static PyObject * AsInteger(ObjectObject *self);
		static PyObject * AsOctal(ObjectObject *self);
		static PyObject * AsHex(ObjectObject *self);
		static int NonZero(ObjectObject *self);
		static PyObject * RichCompare(PyObject *self, PyObject *other, int op);
		static long Hash(ObjectObject *self);

		static PyObject * GetID(ObjectObject *self);
		static PyObject * GetName(ObjectObject *self);
		static PyObject * Exists(ObjectObject *self);
		static PyObject * Clone(ObjectObject *self, PyObject *arg); // METH_O
		static PyObject * GetArchetype(ObjectObject *self);
		static PyObject * SetArchetype(ObjectObject *self, PyObject *arg); // METH_O
		static PyObject * IsArchetype(ObjectObject *self);
		static PyObject * IsMetaProperty(ObjectObject *self);
	
		static const char * Name;
		static const char * DocStrings[];
		static const PyNumberMethods NumMethods;
		static const PyMethodDef Methods[];
		static PyTypeObject Type;
	};

	class LinkObject : public PyObject
	{
	public:
		link m_link;
		int m_source, m_dest;
		short m_flavor;
		
		static void InitModule(PyObject *module);
		static PyObject * New(link l);
		static PyObject * Construct(PyTypeObject *type, PyObject *args, PyObject *kwds);
		static PyObject * Pickle(PyObject *nil, LinkObject *self);

		static void Dealloc(LinkObject *self);
		static PyObject * Repr(LinkObject *self);
		static PyObject * Str(LinkObject *self);
		static PyObject * AsInteger(LinkObject *self);
		static PyObject * AsOctal(LinkObject *self);
		static PyObject * AsHex(LinkObject *self);
		static int NonZero(LinkObject *self);
		static PyObject * RichCompare(PyObject *self, PyObject *other, int op);
		static long Hash(LinkObject *self);

		static PyObject * ID(LinkObject *self);
		static PyObject * To(LinkObject *self);
		static PyObject * From(LinkObject *self);
		static PyObject * Flavor(LinkObject *self);
		static PyObject * GetData(LinkObject *self);

		static const char * Name;
		static const char * DocStrings[];
		static const PyNumberMethods NumMethods;
		static const PyMethodDef Methods[];
		static PyTypeObject Type;
	};
	
	class LinkIterObject : public PyObject
	{
	public:
		ILinkQuery * m_query;
		
		static void InitModule(PyObject *module);
		// "steals" the reference to q
		static PyObject * New(ILinkQuery *q);
		//static PyObject * Construct(PyTypeObject *type, PyObject *args, PyObject *kwds);
		//static PyObject * Pickle(PyObject *, LinkIterObject *self);

		static void Dealloc(LinkIterObject *self);
		static PyObject * Repr(LinkIterObject *self);
		//static long Hash(LinkIterObject *self);

		static PyObject * Iter(LinkIterObject *self);
		static PyObject * Next(LinkIterObject *self);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject Type;
	};

	class TimerObject : public PyObject
	{
	public:
		tScrTimer m_handle;

		static void InitModule(PyObject *module);
		static PyObject * New(tScrTimer h);
		static PyObject * Construct(PyTypeObject *type, PyObject *args, PyObject *kwds);
		static PyObject * Pickle(PyObject *nil, TimerObject *self);

		static void Dealloc(TimerObject *self);
		static PyObject * Repr(TimerObject *self);
		static PyObject * AsInteger(TimerObject *self);
		static PyObject * AsOctal(TimerObject *self);
		static PyObject * AsHex(TimerObject *self);
		static int NonZero(TimerObject *self);
		static PyObject * RichCompare(PyObject *self, PyObject *other, int op);
		static long Hash(TimerObject *self);

		static PyObject * KillTimer(TimerObject *self);

		static const char * Name;
		static const char * DocStrings[];
		static const PyNumberMethods NumMethods;
		static const PyMethodDef Methods[];
		static PyTypeObject Type;
};
	
} // namespace PyNumbra
