/******************************************************************************
 *    ScriptPersistence.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/types.h>
#include <lg/scrmanagers.h>

#include "pyutils.h"

extern IScriptMan *g_pScriptManager;

namespace PyNumbra
{

	static const char *_pszPersistentName = "_PickledScript";


	PyObject * GetScriptPersistence(PyObject *obClass, int iObjId)
	{
		sScrDatumTag tag;
		tag.objId = iObjId;
		tag.pszName = _pszPersistentName;
		PyObject *obClassName = GetObjectName(obClass);
		if (!obClassName)
		{
			return NULL;
		}
		tag.pszClass = PyString_AsString(obClassName);
		cMultiParm data;
		g_pScriptManager->GetScriptData(&tag, static_cast<sMultiParm*>(&data));
		Py_DECREF(obClassName);
		PyObject *ret = NULL;
		if (data.type == kMT_String)
			ret = MultiParmToObject(data);
		data.Free();
		return ret;
	}

	int SetScriptPersistence(PyObject *obPersistence, PyObject *obClass, int iObjId)
	{
		cMultiParm data;
		MultiParmFromObject(data, obPersistence);
		if (data.type != kMT_String)
		{
			data.Free();
			return -1;
		}
		sScrDatumTag tag;
		tag.objId = iObjId;
		tag.pszName = _pszPersistentName;
		PyObject *obClassName = GetObjectName(obClass);
		if (!obClassName)
		{
			data.Free();
			return -1;
		}
		tag.pszClass = PyString_AsString(obClassName);
		int ret = g_pScriptManager->SetScriptData(&tag, static_cast<sMultiParm*>(&data));
		data.Free();
		Py_DECREF(obClassName);
		return ret;
	}
	
	PyObject * pickleReconstruct(PyObject *obPersistence)
	{
		PyObject *module = PyImport_ImportModule("cPickle");
		if (!module)
		{
			module = PyImport_ImportModule("pickle");
			if (!module)
				return NULL;
		}
		PyObject *unpickler = PyObject_GetAttrString(module, "loads");
		if (!unpickler)
		{
			Py_DECREF(module);
			return NULL;
		}
		PyObject *obScript = PyObject_CallFunction(unpickler, "(O)", obPersistence);
		Py_DECREF(module);
		return obScript;
	}

	PyObject * pickleDeconstruct(PyObject *obScript)
	{
		PyObject *module = PyImport_ImportModule("cPickle");
		if (!module)
		{
			module = PyImport_ImportModule("pickle");
			if (!module)
				return NULL;
		}
		PyObject *pickler = PyObject_GetAttrString(module, "dumps");
		if (!pickler)
		{
			Py_DECREF(module);
			return NULL;
		}
		PyObject *obPersistence = PyObject_CallFunction(pickler, "(O)", obScript);
		Py_DECREF(module);
		return obPersistence;
	}

}
