/******************************************************************************
 *    ScriptServices.h
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/scrmanagers.h>

extern IScriptMan *g_pScriptManager;

namespace PyNumbra
{

	extern "C" __declspec(dllexport) 
	PyObject * initScriptServices(void);

	template<class _IFace>
	struct ScriptServiceObject : public PyObject
	{
		//PyObject_HEAD
		_IFace * m_pIFace;
	};


	template<class _IFace>
	class ScriptService 
	{
	public:
		typedef ScriptServiceObject<_IFace>	_ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }

		static _ObjectType * NewObject(PyObject *);
		static void Dealloc(_ObjectType *self);

		static PyObject * Repr(_ObjectType *self);
		static int Print(_ObjectType *self, FILE *fp, int flags);
		static int Compare(_ObjectType *self, _ObjectType *other);
		
		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// ActReactService
	template<>
	class ScriptService<IActReactSrv> 
	{
	public:
		typedef ScriptServiceObject<IActReactSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * React(_ObjectType *self, PyObject *args);
		static PyObject * Stimulate(_ObjectType *self, PyObject *args);
		static PyObject * GetReactionNamed(_ObjectType *self, PyObject *args);
		static PyObject * GetReactionName(_ObjectType *self, PyObject *args);
		static PyObject * SubscribeToStimulus(_ObjectType *self, PyObject *args);
		static PyObject * UnsubscribeToStimulus(_ObjectType *self, PyObject *args);
		static PyObject * BeginContact(_ObjectType *self, PyObject *args);
		static PyObject * EndContact(_ObjectType *self, PyObject *args);
		static PyObject * SetSingleSensorContact(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// AIScriptService
	template<>
	class ScriptService<IAIScrSrv> 
	{
	public:
		typedef ScriptServiceObject<IAIScrSrv> _ObjectType;

		static void InitModule(PyObject *);
		static _ObjectType * NewObject(PyObject *);

		static PyObject * MakeGotoObjLoc(_ObjectType *self, PyObject *args);
		static PyObject * MakeFrobObj2(_ObjectType *self, PyObject *args);
		static PyObject * MakeFrobObj(_ObjectType *self, PyObject *args);
		static PyObject * GetAlertLevel(_ObjectType *self, PyObject *args);
		static PyObject * SetMinimumAlert(_ObjectType *self, PyObject *args);
		static PyObject * ClearGoals(_ObjectType *self, PyObject *args);
		static PyObject * SetScriptFlags(_ObjectType *self, PyObject *args);
		static PyObject * ClearAlertness(_ObjectType *self, PyObject *args);
		static PyObject * Signal(_ObjectType *self, PyObject *args);
		static PyObject * StartConversation(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// AnimTextureService
	template<>
	class ScriptService<IAnimTextureSrv> 
	{
	public:
		typedef ScriptServiceObject<IAnimTextureSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * ChangeTexture(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// BowService
	template<>
	class ScriptService<IBowSrv> 
	{
	public:
		typedef ScriptServiceObject<IBowSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * Equip(_ObjectType *self);
		static PyObject * UnEquip(_ObjectType *self);
		static PyObject * IsEquipped(_ObjectType *self);
		static PyObject * StartAttack(_ObjectType *self);
		static PyObject * FinishAttack(_ObjectType *self);
		static PyObject * AbortAttack(_ObjectType *self);
		static PyObject * SetArrow(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

#if (_DARKGAME == 2)
// CameraService
	template<>
	class ScriptService<ICameraSrv> 
	{
	public:
		typedef ScriptServiceObject<ICameraSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * StaticAttach(_ObjectType *self, PyObject *args);
		static PyObject * DynamicAttach(_ObjectType *self, PyObject *args);
		static PyObject * CameraReturn(_ObjectType *self, PyObject *args);
		static PyObject * ForceCameraReturn(_ObjectType *self);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};
#endif

// ContainerService
	template<>
	class ScriptService<IContainSrv> 
	{
	public:
		typedef ScriptServiceObject<IContainSrv> _ObjectType;

		static void InitModule(PyObject *);
		static _ObjectType * NewObject(PyObject *);

		static PyObject * Add(_ObjectType *self, PyObject *args);
		static PyObject * Remove(_ObjectType *self, PyObject *args);
		static PyObject * MoveAllContents(_ObjectType *self, PyObject *args);
#if (_DARKGAME == 2)
		static PyObject * StackAdd(_ObjectType *self, PyObject *args);
		static PyObject * IsHeld(_ObjectType *self, PyObject *args);
#endif

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// DamageService
	template<>
	class ScriptService<IDamageSrv> 
	{
	public:
		typedef ScriptServiceObject<IDamageSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * Damage(_ObjectType *self, PyObject *args);
		static PyObject * Slay(_ObjectType *self, PyObject *args);
		static PyObject * Resurrect(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// DarkGameService
	template<>
	class ScriptService<IDarkGameSrv> 
	{
	public:
		typedef ScriptServiceObject<IDarkGameSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * KillPlayer(_ObjectType *self);
		static PyObject * EndMission(_ObjectType *self);
		static PyObject * FadeToBlack(_ObjectType *self, PyObject *args);
#if (_DARKGAME == 2)
		static PyObject * FoundObject(_ObjectType *self, PyObject *args);
		static PyObject * ConfigIsDefined(_ObjectType *self, PyObject *args);
		static PyObject * ConfigGetInt(_ObjectType *self, PyObject *args);
		static PyObject * ConfigGetFloat(_ObjectType *self, PyObject *args);
		static PyObject * BindingGetFloat(_ObjectType *self, PyObject *args);
		static PyObject * GetAutomapLocationVisited(_ObjectType *self, PyObject *args);
		static PyObject * SetAutomapLocationVisited(_ObjectType *self, PyObject *args);
#endif

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// DarkUIService
	template<>
	class ScriptService<IDarkUISrv> 
	{
	public:
		typedef ScriptServiceObject<IDarkUISrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * TextMessage(_ObjectType *self, PyObject *args);
		static PyObject * ReadBook(_ObjectType *self, PyObject *args);
		static PyObject * InvItem(_ObjectType *self);
		static PyObject * InvWeapon(_ObjectType *self);
		static PyObject * InvSelect(_ObjectType *self, PyObject *args);
		static PyObject * IsCommandBound(_ObjectType *self, PyObject *args);
		static PyObject * DescribeKeyBinding(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// DataService
	template<>
	class ScriptService<IDataSrv> 
	{
	public:
		typedef ScriptServiceObject<IDataSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * GetString(_ObjectType *self, PyObject *args);
		static PyObject * GetObjString(_ObjectType *self, PyObject *args);
		static PyObject * DirectRand(_ObjectType *self);
		static PyObject * RandInt(_ObjectType *self, PyObject *args);
		static PyObject * RandFlt0to1(_ObjectType *self);
		static PyObject * RandFltNeg1to1(_ObjectType *self);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// DebugScriptService
	template<>
	class ScriptService<IDebugScrSrv> 
	{
	public:
		typedef ScriptServiceObject<IDebugScrSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * MPrint(_ObjectType *self, PyObject *args);
		static PyObject * Command(_ObjectType *self, PyObject *args);
		static PyObject * Break(_ObjectType *self);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// DoorService
	template<>
	class ScriptService<IDoorSrv> 
	{
	public:
		typedef ScriptServiceObject<IDoorSrv> _ObjectType;

		static void InitModule(PyObject *);
		static _ObjectType * NewObject(PyObject *);

		static PyObject * CloseDoor(_ObjectType *self, PyObject *args);
		static PyObject * OpenDoor(_ObjectType *self, PyObject *args);
		static PyObject * GetDoorState(_ObjectType *self, PyObject *args);
		static PyObject * ToggleDoor(_ObjectType *self, PyObject *args);
#if (_DARKGAME == 2)
		static PyObject * SetBlocking(_ObjectType *self, PyObject *args);
		static PyObject * GetSoundBlocking(_ObjectType *self, PyObject *args);
#endif

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// DarkInvService
	template<>
	class ScriptService<IDarkInvSrv> 
	{
	public:
		typedef ScriptServiceObject<IDarkInvSrv> _ObjectType;

		static void InitModule(PyObject *);
		static _ObjectType * NewObject(PyObject *);

		static PyObject * CapabilityControl(_ObjectType *self, PyObject *args);
		static PyObject * AddSpeedControl(_ObjectType *self, PyObject *args);
		static PyObject * RemoveSpeedControl(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// DarkPowerupsService
	template<>
	class ScriptService<IDarkPowerupsSrv> 
	{
	public:
		typedef ScriptServiceObject<IDarkPowerupsSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * TriggerWorldFlash(_ObjectType *self, PyObject *args);
		static PyObject * ObjTryDeploy(_ObjectType *self, PyObject *args);
		static PyObject * CleanseBlood(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// KeyService
	template<>
	class ScriptService<IKeySrv> 
	{
	public:
		typedef ScriptServiceObject<IKeySrv> _ObjectType;

		static void InitModule(PyObject *);
		static _ObjectType * NewObject(PyObject *);

		static PyObject * TryToUseKey(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// LightScriptService
	template<>
	class ScriptService<ILightScrSrv> 
	{
	public:
		typedef ScriptServiceObject<ILightScrSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * Set(_ObjectType *self, PyObject *args);
		static PyObject * SetMode(_ObjectType *self, PyObject *args);
		static PyObject * Activate(_ObjectType *self, PyObject *args);
		static PyObject * Deactivate(_ObjectType *self, PyObject *args);
		static PyObject * Subscribe(_ObjectType *self, PyObject *args);
		static PyObject * Unsubscribe(_ObjectType *self, PyObject *args);
		static PyObject * GetMode(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// LinkService
	template<>
	class ScriptService<ILinkSrv> 
	{
	public:
		typedef ScriptServiceObject<ILinkSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * Create(_ObjectType *self, PyObject *args);
		static PyObject * Destroy(_ObjectType *self, PyObject *args);
		static PyObject * AnyExist(_ObjectType *self, PyObject *args);
		static PyObject * GetAll(_ObjectType *self, PyObject *args);
		static PyObject * GetOne(_ObjectType *self, PyObject *args);
		static PyObject * BroadcastOnAllLinks(_ObjectType *self, PyObject *args);
		static PyObject * CreateMany(_ObjectType *self, PyObject *args);
		static PyObject * DestroyMany(_ObjectType *self, PyObject *args);
		static PyObject * GetAllInherited(_ObjectType *self, PyObject *args);
		static PyObject * GetAllInheritedSingle(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// LinkToolsService
	template<>
	class ScriptService<ILinkToolsSrv> 
	{
	public:
		typedef ScriptServiceObject<ILinkToolsSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * LinkKindNamed(_ObjectType *self, PyObject *args);
		static PyObject * LinkKindName(_ObjectType *self, PyObject *args);
		static PyObject * LinkGet(_ObjectType *self, PyObject *args);
		static PyObject * LinkGetData(_ObjectType *self, PyObject *args);
		static PyObject * LinkSetData(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// LockService
	template<>
	class ScriptService<ILockSrv> 
	{
	public:
		typedef ScriptServiceObject<ILockSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * IsLocked(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

#if (_DARKGAME == 3)
// NetworkingService
	template<>
	class ScriptService<INetworkingSrv> 
	{
	public:
		typedef ScriptServiceObject<INetworkingSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * Broadcast(_ObjectType *self, PyObject *args);
		static PyObject * SendToProxy(_ObjectType *self, PyObject *args);
		static PyObject * TakeOver(_ObjectType *self, PyObject *args);
		static PyObject * GiveTo(_ObjectType *self, PyObject *args);
		static PyObject * IsPlayer(_ObjectType *self, PyObject *args);
		static PyObject * IsMultiplayer(_ObjectType *self);
		static PyObject * SetProxyOneShotTimer(_ObjectType *self, PyObject *args);
		static PyObject * FirstPlayer(_ObjectType *self);
		static PyObject * NextPlayer(_ObjectType *self);
		static PyObject * Suspend(_ObjectType *self);
		static PyObject * Resume(_ObjectType *self);
		static PyObject * HostedHere(_ObjectType *self, PyObject *args);
		static PyObject * IsProxy(_ObjectType *self, PyObject *args);
		static PyObject * LocalOnly(_ObjectType *self, PyObject *args);
		static PyObject * IsNetworking(_ObjectType *self);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};
#endif

// ObjectService
	template<>
	class ScriptService<IObjectSrv> 
	{
	public:
		typedef ScriptServiceObject<IObjectSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * BeginCreate(_ObjectType *self, PyObject *args);
		static PyObject * EndCreate(_ObjectType *self, PyObject *args);
		static PyObject * Create(_ObjectType *self, PyObject *args);
		static PyObject * Destroy(_ObjectType *self, PyObject *args);
		static PyObject * Exists(_ObjectType *self, PyObject *args);
		static PyObject * SetName(_ObjectType *self, PyObject *args);
		static PyObject * GetName(_ObjectType *self, PyObject *args);
		static PyObject * Named(_ObjectType *self, PyObject *args);
		static PyObject * AddMetaProperty(_ObjectType *self, PyObject *args);
		static PyObject * RemoveMetaProperty(_ObjectType *self, PyObject *args);
		static PyObject * HasMetaProperty(_ObjectType *self, PyObject *args);
		static PyObject * InheritsFrom(_ObjectType *self, PyObject *args);
		static PyObject * IsTransient(_ObjectType *self, PyObject *args);
		static PyObject * SetTransience(_ObjectType *self, PyObject *args);
		static PyObject * Position(_ObjectType *self, PyObject *args);
		static PyObject * Facing(_ObjectType *self, PyObject *args);
		static PyObject * Teleport(_ObjectType *self, PyObject *args);
#if (_DARKGAME == 2) || (_DARKGAME == 3)
		static PyObject * IsPositionValid(_ObjectType *self, PyObject *args);
#endif
#if (_DARKGAME == 2)
		static PyObject * FindClosestObjectNamed(_ObjectType *self, PyObject *args);
#endif
		static PyObject * AddMetaPropertyToMany(_ObjectType *self, PyObject *args);
		static PyObject * RemoveMetaPropertyFromMany(_ObjectType *self, PyObject *args);
		static PyObject * RenderedThisFrame(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// ParticleGroupService
	template<>
	class ScriptService<IPGroupSrv> 
	{
	public:
		typedef ScriptServiceObject<IPGroupSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * SetActive(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// PhysicsService
	template<>
	class ScriptService<IPhysSrv> 
	{
	public:
		typedef ScriptServiceObject<IPhysSrv> _ObjectType;

		static void InitModule(PyObject *);
		static _ObjectType * NewObject(PyObject *);

		static PyObject * SubscribeMsg(_ObjectType *self, PyObject *args);
		static PyObject * UnsubscribeMsg(_ObjectType *self, PyObject *args);
		static PyObject * LaunchProjectile(_ObjectType *self, PyObject *args);
		static PyObject * SetVelocity(_ObjectType *self, PyObject *args);
		static PyObject * GetVelocity(_ObjectType *self, PyObject *args);
#if (_DARKGAME == 2)
		static PyObject * ControlVelocity(_ObjectType *self, PyObject *args);
		static PyObject * StopControlVelocity(_ObjectType *self, PyObject *args);
#endif
		static PyObject * SetGravity(_ObjectType *self, PyObject *args);
		static PyObject * GetGravity(_ObjectType *self, PyObject *args);
#if (_DARKGAME != 1)
		static PyObject * HasPhysics(_ObjectType *self, PyObject *args);
		static PyObject * IsSphere(_ObjectType *self, PyObject *args);
		static PyObject * IsOBB(_ObjectType *self, PyObject *args);
		static PyObject * ControlCurrentLocation(_ObjectType *self, PyObject *args);
		static PyObject * ControlCurrentRotation(_ObjectType *self, PyObject *args);
		static PyObject * ControlCurrentPosition(_ObjectType *self, PyObject *args);
		static PyObject * DeregisterModel(_ObjectType *self, PyObject *args);
		static PyObject * PlayerMotionSetOffset(_ObjectType *self, PyObject *args);
		static PyObject * Activate(_ObjectType *self, PyObject *args);
		static PyObject * ValidPos(_ObjectType *self, PyObject *args);
#endif

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// PickLockService
	template<>
	class ScriptService<IPickLockSrv> 
	{
	public:
		typedef ScriptServiceObject<IPickLockSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * Ready(_ObjectType *self, PyObject *args);
		static PyObject * UnReady(_ObjectType *self, PyObject *args);
		static PyObject * StartPicking(_ObjectType *self, PyObject *args);
		static PyObject * FinishPicking(_ObjectType *self, PyObject *args);
		static PyObject * CheckPick(_ObjectType *self, PyObject *args);
		static PyObject * DirectMotion(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// PlayerLimbsService
	template<>
	class ScriptService<IPlayerLimbsSrv> 
	{
	public:
		typedef ScriptServiceObject<IPlayerLimbsSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * Equip(_ObjectType *self, PyObject *args);
		static PyObject * UnEquip(_ObjectType *self, PyObject *args);
		static PyObject * StartUse(_ObjectType *self, PyObject *args);
		static PyObject * FinishUse(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// PropertyService
	template<>
	class ScriptService<IPropertySrv> 
	{
	public:
		typedef ScriptServiceObject<IPropertySrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * Get(_ObjectType *self, PyObject *args);
		static PyObject * Set(_ObjectType *self, PyObject *args);
#if (_DARKGAME == 2) || (_DARKGAME == 3)
		static PyObject * SetLocal(_ObjectType *self, PyObject *args);
#endif
		static PyObject * Add(_ObjectType *self, PyObject *args);
		static PyObject * Remove(_ObjectType *self, PyObject *args);
		static PyObject * CopyFrom(_ObjectType *self, PyObject *args);
		static PyObject * Possessed(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// PuppetService
	template<>
	class ScriptService<IPuppetSrv> 
	{
	public:
		typedef ScriptServiceObject<IPuppetSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * PlayMotion(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// QuestService
	template<>
	class ScriptService<IQuestSrv> 
	{
	public:
		typedef ScriptServiceObject<IQuestSrv> _ObjectType;

		static void InitModule(PyObject *);
		static _ObjectType * NewObject(PyObject *);

		static PyObject * SubscribeMsg(_ObjectType *self, PyObject *args);
		static PyObject * UnsubscribeMsg(_ObjectType *self, PyObject *args);
		static PyObject * Set(_ObjectType *self, PyObject *args);
		static PyObject * Get(_ObjectType *self, PyObject *args);
		static PyObject * Exists(_ObjectType *self, PyObject *args);
		static PyObject * Delete(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// SoundScriptService
	template<>
	class ScriptService<ISoundScrSrv> 
	{
	public:
		typedef ScriptServiceObject<ISoundScrSrv> _ObjectType;

		static void InitModule(PyObject *);
		static _ObjectType * NewObject(PyObject *);

		static PyObject * PlayLoc(_ObjectType *self, PyObject *args);
		static PyObject * PlayObj(_ObjectType *self, PyObject *args);
		static PyObject * Play(_ObjectType *self, PyObject *args);
		static PyObject * PlayAmbient(_ObjectType *self, PyObject *args);
		static PyObject * PlaySchemaLoc(_ObjectType *self, PyObject *args);
		static PyObject * PlaySchemaObj(_ObjectType *self, PyObject *args);
		static PyObject * PlaySchema(_ObjectType *self, PyObject *args);
		static PyObject * PlaySchemaAmbient(_ObjectType *self, PyObject *args);
		static PyObject * PlayEnvSchema(_ObjectType *self, PyObject *args);
		static PyObject * PlayVoiceOver(_ObjectType *self, PyObject *args);
		static PyObject * Halt(_ObjectType *self, PyObject *args);
		static PyObject * HaltSchema(_ObjectType *self, PyObject *args);
		static PyObject * HaltSpeech(_ObjectType *self, PyObject *args);
		static PyObject * PreLoad(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// WeaponService
	template<>
	class ScriptService<IWeaponSrv> 
	{
	public:
		typedef ScriptServiceObject<IWeaponSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * Equip(_ObjectType *self, PyObject *args);
		static PyObject * UnEquip(_ObjectType *self, PyObject *args);
		static PyObject * IsEquipped(_ObjectType *self, PyObject *args);
		static PyObject * StartAttack(_ObjectType *self, PyObject *args);
		static PyObject * FinishAttack(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

#if (_DARKGAME == 3)
// ShockGameService
	template<>
	class ScriptService<IShockGameSrv> 
	{
	public:
		typedef ScriptServiceObject<IShockGameSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * DestroyCursorObj(_ObjectType *self);
		static PyObject * DestroyInvObj(_ObjectType *self, PyObject *args);
		static PyObject * HideInvObj(_ObjectType *self, PyObject *args);
		static PyObject * SetPlayerPsiPoints(_ObjectType *self, PyObject *args);
		static PyObject * GetPlayerPsiPoints(_ObjectType *self);
		static PyObject * AttachCamera(_ObjectType *self, PyObject *args);
		static PyObject * CutSceneModeOn(_ObjectType *self, PyObject *args);
		static PyObject * CutSceneModeOff(_ObjectType *self);
		static PyObject * CreatePlayerPuppet(_ObjectType *self, PyObject *args);
		static PyObject * DestroyPlayerPuppet(_ObjectType *self);
		static PyObject * Replicator(_ObjectType *self, PyObject *args);
		static PyObject * Container(_ObjectType *self, PyObject *args);
		static PyObject * YorN(_ObjectType *self, PyObject *args);
		static PyObject * Keypad(_ObjectType *self, PyObject *args);
		static PyObject * HRM(_ObjectType *self, PyObject *args);
		static PyObject * TechTool(_ObjectType *self, PyObject *args);
		static PyObject * UseLog(_ObjectType *self, PyObject *args);
		static PyObject * TriggerLog(_ObjectType *self, PyObject *args);
		static PyObject * FindLogData(_ObjectType *self, PyObject *args);
		static PyObject * PayNanites(_ObjectType *self, PyObject *args);
		static PyObject * OverlayChange(_ObjectType *self, PyObject *args);
		static PyObject * Equipped(_ObjectType *self, PyObject *args);
		static PyObject * LevelTransport(_ObjectType *self, PyObject *args);
		static PyObject * CheckLocked(_ObjectType *self, PyObject *args);
		static PyObject * AddText(_ObjectType *self, PyObject *args);
		static PyObject * AddTranslatableText(_ObjectType *self, PyObject *args);
		static PyObject * AmmoLoad(_ObjectType *self, PyObject *args);
		static PyObject * GetClip(_ObjectType *self, PyObject *args);
		static PyObject * AddExp(_ObjectType *self, PyObject *args);
		static PyObject * HasTrait(_ObjectType *self, PyObject *args);
		static PyObject * HasImplant(_ObjectType *self, PyObject *args);
		static PyObject * HealObj(_ObjectType *self, PyObject *args);
		static PyObject * OverlaySetObj(_ObjectType *self, PyObject *args);
		static PyObject * Research(_ObjectType *self);
		static PyObject * GetArchetypeName(_ObjectType *self, PyObject *args);
		static PyObject * OverlayOn(_ObjectType *self, PyObject *args);
		static PyObject * FindSpawnPoint(_ObjectType *self, PyObject *args);
		static PyObject * CountEcoMatching(_ObjectType *self, PyObject *args);
		static PyObject * GetStat(_ObjectType *self, PyObject *args);
		static PyObject * GetSelectedObj(_ObjectType *self);
		static PyObject * AddInvObj(_ObjectType *self, PyObject *args);
		static PyObject * RecalcStats(_ObjectType *self, PyObject *args);
		static PyObject * PlayVideo(_ObjectType *self, PyObject *args);
		static PyObject * ClearRadiation(_ObjectType *self);
		static PyObject * SetPlayerVolume(_ObjectType *self, PyObject *args);
		static PyObject * RandRange(_ObjectType *self, PyObject *args);
		static PyObject * LoadCursor(_ObjectType *self, PyObject *args);
		static PyObject * AddSpeedControl(_ObjectType *self, PyObject *args);
		static PyObject * RemoveSpeedControl(_ObjectType *self, PyObject *args);
		static PyObject * PreventSwap(_ObjectType *self);
		static PyObject * GetDistantSelectedObj(_ObjectType *self);
		static PyObject * Equip(_ObjectType *self, PyObject *args);
		static PyObject * OverlayChangeObj(_ObjectType *self, PyObject *args);
		static PyObject * SetObjState(_ObjectType *self, PyObject *args);
		static PyObject * RadiationHack(_ObjectType *self);
		static PyObject * DestroyAllByName(_ObjectType *self, PyObject *args);
		static PyObject * AddTextObjProp(_ObjectType *self, PyObject *args);
		static PyObject * DisableAlarmGlobal(_ObjectType *self);
		static PyObject * Frob(_ObjectType *self, PyObject *args);
		static PyObject * TweqAllByName(_ObjectType *self, PyObject *args);
		static PyObject * SetExplored(_ObjectType *self, PyObject *args);
		static PyObject * RemoveFromContainer(_ObjectType *self, PyObject *args);
		static PyObject * ActivateMap(_ObjectType *self);
		static PyObject * SimTime(_ObjectType *self);
		static PyObject * StartFadeIn(_ObjectType *self, PyObject *args);
		static PyObject * StartFadeOut(_ObjectType *self, PyObject *args);
		static PyObject * GrantPsiPower(_ObjectType *self, PyObject *args);
		static PyObject * ResearchConsume(_ObjectType *self, PyObject *args);
		static PyObject * PlayerMode(_ObjectType *self, PyObject *args);
		static PyObject * EndGame(_ObjectType *self);
		static PyObject * AllowDeath(_ObjectType *self);
		static PyObject * AddAlarm(_ObjectType *self, PyObject *args);
		static PyObject * RemoveAlarm(_ObjectType *self);
		static PyObject * GetHazardResistance(_ObjectType *self, PyObject *args);
		static PyObject * GetBurnDmg(_ObjectType *self);
		static PyObject * PlayerGun(_ObjectType *self);
		static PyObject * IsPsiActive(_ObjectType *self, PyObject *args);
		static PyObject * PsiRadarScan(_ObjectType *self);
		static PyObject * PseudoProjectile(_ObjectType *self, PyObject *args);
		static PyObject * WearArmor(_ObjectType *self, PyObject *args);
		static PyObject * SetModify(_ObjectType *self, PyObject *args);
		static PyObject * Censored(_ObjectType *self);
		static PyObject * DebriefMode(_ObjectType *self, PyObject *args);
		static PyObject * TlucTextAdd(_ObjectType *self, PyObject *args);
		static PyObject * Mouse(_ObjectType *self, PyObject *args);
		static PyObject * RefreshInv(_ObjectType *self);
		static PyObject * TreasureTable(_ObjectType *self, PyObject *args);
		static PyObject * OverlayGetObj(_ObjectType *self);
		static PyObject * VaporizeInv(_ObjectType *self);
		static PyObject * ShutoffPsi(_ObjectType *self);
		static PyObject * SetQBHacked(_ObjectType *self, PyObject *args);
		static PyObject * GetPlayerMaxPsiPoints(_ObjectType *self);
		static PyObject * SetLogTime(_ObjectType *self, PyObject *args);
		static PyObject * AddTranslatableTextInt(_ObjectType *self, PyObject *args);
		static PyObject * ZeroControls(_ObjectType *self, PyObject *args);
		static PyObject * SetSelectedPsiPower(_ObjectType *self, PyObject *args);
		static PyObject * ValidGun(_ObjectType *self, PyObject *args);
		static PyObject * AddTranslatableTextIndexInt(_ObjectType *self, PyObject *args);
		static PyObject * IsAlarmActive(_ObjectType *self);
		static PyObject * SlayAllByName(_ObjectType *self, PyObject *args);
		static PyObject * NoMove(_ObjectType *self, PyObject *args);
		static PyObject * PlayerModeSimple(_ObjectType *self, PyObject *args);
		static PyObject * UpdateMovingTerrainVelocity(_ObjectType *self, PyObject *args);
		static PyObject * MouseCursor(_ObjectType *self);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// ShockObjService
	template<>
	class ScriptService<IShockObjSrv> 
	{
	public:
		typedef ScriptServiceObject<IShockObjSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * FindScriptDonor(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// ShockPsiService
	template<>
	class ScriptService<IShockPsiSrv> 
	{
	public:
		typedef ScriptServiceObject<IShockPsiSrv> _ObjectType;

		static void InitModule(PyObject *);
		static _ObjectType * NewObject(PyObject *);

		static PyObject * OnDeactivate(_ObjectType *self, PyObject *args);
		static PyObject * GetActiveTime(_ObjectType *self, PyObject *args);
		static PyObject * IsOverloaded(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// ShockAIService
	template<>
	class ScriptService<IShockAISrv> 
	{
	public:
		typedef ScriptServiceObject<IShockAISrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * Stun(_ObjectType *self, PyObject *args);
		static PyObject * IsStunned(_ObjectType *self, PyObject *args);
		static PyObject * UnStun(_ObjectType *self, PyObject *args);
		static PyObject * Freeze(_ObjectType *self, PyObject *args);
		static PyObject * UnFreeze(_ObjectType *self, PyObject *args);
		static PyObject * NotifyEnterTripwire(_ObjectType *self, PyObject *args);
		static PyObject * NotifyExitTripwire(_ObjectType *self, PyObject *args);
		static PyObject * ObjectLocked(_ObjectType *self, PyObject *args);
		static PyObject * ValidateSpawn(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};

// ShockWeaponService
	template<>
	class ScriptService<IShockWeaponSrv> 
	{
	public:
		typedef ScriptServiceObject<IShockWeaponSrv> _ObjectType;

		static void InitModule(PyObject *)
			{ PyType_Ready(&TypeInfo); }
		static _ObjectType * NewObject(PyObject *);

		static PyObject * SetWeaponModel(_ObjectType *self, PyObject *args);
		static PyObject * GetWeaponModel(_ObjectType *self);
		static PyObject * TargetScan(_ObjectType *self, PyObject *args);
		static PyObject * Home(_ObjectType *self, PyObject *args);
		static PyObject * DestroyMelee(_ObjectType *self, PyObject *args);

		static const char * Name;
		static const char * DocStrings[];
		static const PyMethodDef Methods[];
		static PyTypeObject TypeInfo;
	};
#endif // (_DARKGAME == 3)

}

