/******************************************************************************
 *    interface.h
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#ifndef _LG_INTERFACE_H
#define _LG_INTERFACE_H

#if _MSC_VER > 1000
#pragma once
#endif

#include <exception>
#include <cstdio>

#include <lg/iiddef.h>
#include <lg/scrmanagers.h>
//#include <lg/scrservices.h>

//#pragma interface "interface.h"


class null_pointer : public std::exception
{
public:
	null_pointer() throw() { }
	virtual ~null_pointer() throw() { }

	virtual const char* what() const throw()
	{ return "null pointer"; }
};

class no_interface : public std::exception
{
	const char* ifname;
	char* buf;
public:
	no_interface() throw() : ifname(NULL), buf(NULL) { }
	no_interface(const char* nm) : ifname(nm), buf(NULL) { }
	virtual ~no_interface() throw()
	{
		if (buf)
			delete[] buf;
	}

	virtual const char* what() throw()
	{
		buf = new char[strlen(ifname) + 27];
		sprintf(buf, "interface not available (%s)", ifname);
		return buf;
	}
};

extern IScriptMan * g_pScriptManager;

/**************************************
 * Helper class for interface pointers.
 * This implementation uses a reference count,
 * so it can be instantiated globally.
 */
template<class _IFace, typename _IID = IID_Def<_IFace> >
class CInterface
{
	_IFace* m_pIFace;
	unsigned int m_iRefCount;

public:
	~CInterface() throw()
	{
		if (m_pIFace)
			m_pIFace->Release();
	}
	CInterface() throw()
		: m_pIFace(NULL), m_iRefCount(0)
	{
	}
	explicit CInterface(_IFace* __p) throw()
		: m_pIFace(__p), m_iRefCount(0)
	{
		if (m_pIFace)
		{
			m_pIFace->AddRef();
			m_iRefCount = 1;
		}
	}
	CInterface(CInterface& __x) throw()
		: m_pIFace(__x.m_pIFace), m_iRefCount(0)
	{
		if (m_pIFace)
		{
			m_pIFace->AddRef();
			m_iRefCount = 1;
		}
	}
	/*
	template<class _ExtIFace>
	CInterface(CInterface<_ExtIFace>& __x) throw()
	{
		m_pIFace = static_cast<_IFace*>(__x.m_pIFace);
		if (m_pIFace)
			m_pIFace->AddRef();
	}
	*/
	/*
	CInterface(IScriptMan* pSM) throw(no_interface)
		: m_pIFace(NULL)
	{
		if (E_NOINTERFACE == pSM->QueryInterface(_IID::iid(), reinterpret_cast<void**>(&m_pIFace)))
			throw no_interface(_IID::name());
	}
	*/

	/*
	CInterface& operator=(CInterface& __x) throw()
	{
		m_pIFace = __x.m_pIFace;
		if (m_pIFace)
			m_pIFace->AddRef();
		return *this;
	}
	CInterface& operator=(_IFace* __p) throw()
	{
		m_pIFace = __p;
		return *this;
	}
	*/

	_IFace* Acquire() throw(no_interface)
	{
		if (!m_pIFace)
		{
			if (E_NOINTERFACE == g_pScriptManager->QueryInterface(_IID::iid(), reinterpret_cast<void**>(&m_pIFace)))
				throw no_interface(_IID::name());
		}
		++m_iRefCount;
		return m_pIFace;
	}
	void Release() throw()
	{
		if (--m_iRefCount == 0)
		{
			m_pIFace->Release();
			m_pIFace = NULL;
		}
	}

	operator _IFace* () const throw()
	{
		return m_pIFace;
	}

	_IFace& operator*() const throw()
	{
		//if (!m_pIFace)
		//	throw null_pointer();
		return *m_pIFace;
	}
	_IFace* operator->() const throw()
	{
		//if (!m_pIFace)
		//	throw null_pointer();
		return m_pIFace;
	}
};

/******************************************************
 * Helper class for script service interface pointers.
 */
template<class _IFace, typename _IID = IID_Def<_IFace> >
class CService
{
	_IFace* m_pIFace;
	unsigned int m_iRefCount;

public:
	~CService() throw()
	{
		if (m_pIFace)
			m_pIFace->Release();
	}
	CService() throw()
		: m_pIFace(NULL), m_iRefCount(0)
	{
	}
	explicit CService(_IFace* __p) throw()
		: m_pIFace(__p), m_iRefCount(0)
	{
		if (m_pIFace)
		{
			m_pIFace->AddRef();
			m_iRefCount = 1;
		}
	}
	CService(CService& __x) throw()
		: m_pIFace(__x.m_pIFace), m_iRefCount(0)
	{
		if (m_pIFace)
		{
			m_pIFace->AddRef();
			m_iRefCount = 1;
		}
	}
	/*
	template<class _ExtIFace>
	CService(CService<_ExtIFace>& __x) throw()
	{
		m_pIFace = static_cast<_IFace*>(__x.m_pIFace);
		if (m_pIFace)
			m_pIFace->AddRef();
	}
	*/
	/*
	CService(IScriptMan* pSM) throw(no_interface)
	{
		m_pIFace = static_cast<_IFace*>(pSM->GetService(_IID::iid()));
		if (!m_pIFace)
			throw no_interface(_IID::name());
	}
	*/

	/*
	CService& operator=(CService& __x) throw()
	{
		m_pIFace = __x.m_pIFace;
		if (m_pIFace)
			m_pIFace->AddRef();
		return *this;
	}
	CService& operator=(_IFace* __p) throw()
	{
		m_pIFace = __p;
		return *this;
	}
	*/

	_IFace* Acquire() throw(no_interface)
	{
		if (!m_pIFace)
		{
			if (!(m_pIFace = static_cast<_IFace*>(g_pScriptManager->GetService(_IID::iid()))))
				throw no_interface(_IID::name());
		}
		++m_iRefCount;
		return m_pIFace;
	}
	void Release() throw()
	{
		if (--m_iRefCount == 0)
		{
			m_pIFace->Release();
			m_pIFace = NULL;
		}
	}

	operator _IFace* () const throw()
	{
		return m_pIFace;
	}

	_IFace& operator*() const throw()
	{
		//if (!m_pIFace)
		//	throw null_pointer();
		return *m_pIFace;
	}
	_IFace* operator->() const throw()
	{
		//if (!m_pIFace)
		//	throw null_pointer();
		return m_pIFace;
	}
};

#endif // _LG_INTERFACE_H
